@@ -1,7 +1,11 @@
 package CGI::Auth::FOAF_SSL;
 
+use 5.008001;
+use strict;
+use warnings;
+
 BEGIN {
-	$CGI::Auth::FOAF_SSL::VERSION = '0.05';
+	$CGI::Auth::FOAF_SSL::VERSION = '0.50';
 }
 
 =head1 NAME
@@ -10,7 +14,7 @@ CGI::Auth::FOAF_SSL - Authentication using FOAF+SSL.
 
 =head1 VERSION
 
-0.05
+0.50
 
 =head1 SYNOPSIS
 
@@ -20,7 +24,7 @@ CGI::Auth::FOAF_SSL - Authentication using FOAF+SSL.
  my $cgi  = CGI->new;
  my $auth = CGI::Auth::FOAF_SSL->new_from_cgi($cgi);
  
- print header("text/html");
+ print header('-type' => 'text/html', '-cookie' => $auth->cookie);
  
  if (defined $auth && $auth->is_secure)
  {
@@ -60,6 +64,9 @@ setup:
  SSLVerifyClient optional_no_ca
  SSLVerifyDepth  1
  SSLOptions +StdEnvVars +ExportCertData
+ 
+As of version 0.50, this package no longer uses RDF::Redland, using
+RDF::Trine and RDF::Query instead.
 
 =cut
 
@@ -67,13 +74,15 @@ use Carp;
 use CGI;
 use CGI::Auth::FOAF_SSL::OnlineAccount;
 use CGI::Auth::FOAF_SSL::Agent;
+use CGI::Session;
 use IPC::Open2;
 use LWP::UserAgent;
-use RDF::RDFa::Parser;
-use RDF::Redland;
-use 5.010000;
-use strict;
-use warnings;
+use RDF::RDFa::Parser 0.21;
+use RDF::Query;
+use RDF::Query::Client;
+use RDF::Trine 0.111;
+use RDF::Trine::Serializer::NTriples;
+use WWW::Finger::Fingerpoint;
 
 =head1 CONFIGURATION
 
@@ -106,7 +115,7 @@ BEGIN {
 
 Performs FOAF+SSL authentication on a PEM-encoded key. If authentication is
 completely unsuccessful, returns undef. Otherwise, returns a CGI::Auth::FOAF_SSL
-object. Use C<is_secure> to check if authentication was completely successful.
+object. Use C<is_secure> to check if authentication was I<completely> successful.
 
 You probably want to use C<new_from_cgi> instead.
 
@@ -118,8 +127,28 @@ sub new
 	
 	return undef unless $rv;
 	
-	$rv->verify_certificate;
-	$rv->load_personal_info;
+	my $verified = 0;
+	if (defined $rv->{'subject_alt_names'}->{'URI'})
+	{
+		foreach my $uri (@{ $rv->{'subject_alt_names'}->{'URI'} })
+		{
+			$verified = $rv->verify_certificate_by_uri($uri);
+			last if $verified;
+		}
+	}
+	
+	if (defined $rv->{'subject_alt_names'}->{'EMAIL'}
+	and !$verified)
+	{
+		foreach my $e (@{ $rv->{'subject_alt_names'}->{'EMAIL'} })
+		{
+			$verified = $rv->verify_certificate_by_email($e);
+			last if $verified;
+		}
+	}
+	
+	$rv->load_personal_info
+		if $verified;
 	
 	return $rv;
 }
@@ -144,19 +173,10 @@ sub new_from_cgi
 	
 	# This does work, but is less elegant.
 	my $cert = $ENV{'SSL_CLIENT_CERT'};
-	
+
 	return $class->new($cert);
 }
 
-=item $auth = CGI::Auth::FOAF_SSL->new_smiple($pem_encoded)
-
-Performs FOAF+SSL authentication on a PEM-encoded key. This is faster than the
-usual constructor but performs fewer of the usual checks.
-
-You probably want to use C<new_from_cgi> instead.
-
-=cut
-
 sub new_smiple
 {
 	my $class = shift;
@@ -216,9 +236,14 @@ sub new_smiple
 	close WRITE;
 
 	# Only allow FOAF+SSL certificates.
-	return unless $alt_name =~ /^URI:(.+)$/i;
-	$rv->{'cert_subject_uri'} = $1;
+	$rv->{'subject_alt_names'} = {};
+	while ($alt_name =~ /(?:\s|\,|^)(URI|email):([^\s\,]+)(?:\s|\,|$)/ig)
+	{
+		push @{ $rv->{'subject_alt_names'}->{uc $1} }, $2;
+	}
 	
+	return unless $rv->{'subject_alt_names'};
+
 	# Modulus.
 	$pid = 0;
 	$pid = open2(\*READ, \*WRITE, "$openssl x509 -modulus");
@@ -234,56 +259,39 @@ sub new_smiple
 	return unless $exponent_line =~ /Exponent: (\d+) \(0x([0-9A-F]+)\)/i;
 	$rv->{'cert_exponent_dec'} = $1;
 	$rv->{'cert_exponent_hex'} = $2;	
-	
+
 	bless $rv, $class;
 }
 
-=back
-
-=head1 PUBLIC METHODS
-
-=over 8
-
-=item $auth->verify_certificate
-
-This loads the certificate subject URI and checks that the URI confirms
-the certificate's details. If you constructed the object with C<new> or
-C<new_from_cgi>, then you do not need to call this. It is only useful if
-you constructed the object using C<new_smiple>.
-
-Returns true iff the certificate checks out correctly.
-
-=cut
-
-sub verify_certificate
+sub verify_certificate_by_uri
 {
-	my $rv = shift;
-	return 0 unless defined $rv;
-		
-	$rv->{'cert_subject_model'} = $rv->_get_redland_model($rv->{'cert_subject_uri'});
+	my $rv  = shift;
+	my $uri = shift;
 	
-	my $query_string = sprintf("SELECT ?decExponent ?hexModulus "
-	                          ."WHERE { "
-	                          ."    ?key <http://www.w3.org/ns/auth/cert#identity> <%s> . "
-	                          ."    ?key <http://www.w3.org/ns/auth/rsa#modulus> ?modulus . "
-	                          ."    ?modulus <http://www.w3.org/ns/auth/cert#hex> ?hexModulus . "
-	                          ."    ?key <http://www.w3.org/ns/auth/rsa#public_exponent> ?exponent . "
-	                          ."    ?exponent <http://www.w3.org/ns/auth/cert#decimal> ?decExponent . "
+	my $model = $rv->get_trine_model($uri);
+	
+	my $query_string = sprintf("PREFIX cert: <http://www.w3.org/ns/auth/cert#> "
+	                          ."PREFIX rsa: <http://www.w3.org/ns/auth/rsa#> "
+	                          ."SELECT ?decExponent ?hexModulus "
+	                          ."WHERE "
+	                          ."{ "
+	                          ."    ?key "
+	                          ."        cert:identity <%s> ; "
+	                          ."        rsa:modulus [ cert:hex ?hexModulus ] ; "
+	                          ."        rsa:public_exponent [ cert:decimal ?decExponent ] . "
 	                          ."}",
-	                          $rv->{'cert_subject_uri'});
-	my $query   = RDF::Redland::Query->new(
-		$query_string,
-		RDF::Redland::URI->new($rv->{'cert_subject_uri'}),
-		undef,
-		"sparql");
-	my $results = $rv->{'cert_subject_model'}->query_execute($query);
-	if (!$results->finished)
+	                          $uri);
+	my $query   = RDF::Query->new($query_string, $uri);
+	my $results = $query->execute($model);
+	
+	if (my $row = $results->next)
 	{
-		$rv->{'correct_cert_exponent_dec'} = $results->binding_value(0)->literal_value;
-		$rv->{'correct_cert_modulus_hex'}  = $results->binding_value(1)->literal_value;
+		$rv->{'correct_cert_exponent_dec'} = $row->{'decExponent'}->literal_value;
+		$rv->{'correct_cert_modulus_hex'}  = $row->{'hexModulus'}->literal_value;
 	}
 	
-	return 0 unless $rv->{'correct_cert_modulus_hex'} && $rv->{'correct_cert_exponent_dec'};
+	return 0
+		unless $rv->{'correct_cert_modulus_hex'} && $rv->{'correct_cert_exponent_dec'};
 
 	$rv->{'correct_cert_exponent_dec'} =~ s/[^0-9]//ig;
 	$rv->{'correct_cert_modulus_hex'}  =~ s/[^A-Z0-9]//ig;
@@ -292,31 +300,98 @@ sub verify_certificate
 	if (($rv->{'correct_cert_modulus_hex'} eq $rv->{'cert_modulus_hex'})
 	&&  ($rv->{'correct_cert_exponent_dec'} == $rv->{'cert_exponent_dec'}))
 	{
-		$rv->{validation} = 'cert';
+		$rv->{'validation'} = 'cert';
 		delete $rv->{'correct_cert_exponent_dec'};
 		delete $rv->{'correct_cert_modulus_hex'};
+		
+		$rv->{'cert_subject_uri'}   = $uri;
+		$rv->{'cert_subject_model'} = $model;
+		
 		return 1;
 	}
 
 	return 0;
 }
 
-=item $auth->load_personal_info
+sub verify_certificate_by_email
+{
+	my $rv    = shift;
+	my $email = shift;
+	
+	my $fp = WWW::Finger::Fingerpoint->new($email);
+	
+	return 0
+		unless defined $fp->endpoint and defined $fp->webid;
 
-This loads the certificate subject URI and investigates that entity. If you
-constructed the object with C<new> or C<new_from_cgi>, then you do not need
-to call this. It is only useful if you constructed the object using
-C<new_smiple>.
+	my $query_string = sprintf("PREFIX cert: <http://www.w3.org/ns/auth/cert#> "
+	                          ."PREFIX rsa: <http://www.w3.org/ns/auth/rsa#> "
+	                          ."SELECT ?decExponent ?hexModulus "
+	                          ."WHERE "
+	                          ."{ "
+	                          ."    ?key "
+	                          ."        cert:identity <%s> ; "
+	                          ."        rsa:modulus [ cert:hex ?hexModulus ] ; "
+	                          ."        rsa:public_exponent [ cert:decimal ?decExponent ] . "
+	                          ."}",
+	                          $fp->webid);
+									  
+	my $query   = RDF::Query::Client->new($query_string);
+	my $results = $query->execute($fp->endpoint, {QueryMethod=>'POST'});
+	
+	if (my $row = $results->next)
+	{
+		$rv->{'correct_cert_exponent_dec'} = $row->{'decExponent'}->literal_value;
+		$rv->{'correct_cert_modulus_hex'}  = $row->{'hexModulus'}->literal_value;
+	}
+	
+	return 0
+		unless $rv->{'correct_cert_modulus_hex'} && $rv->{'correct_cert_exponent_dec'};
 
-Returns true iff some personal or account details could be found.
+	$rv->{'correct_cert_exponent_dec'} =~ s/[^0-9]//ig;
+	$rv->{'correct_cert_modulus_hex'}  =~ s/[^A-Z0-9]//ig;
+	$rv->{'correct_cert_modulus_hex'}  = uc($rv->{'correct_cert_modulus_hex'});
 
-=cut
+	if (($rv->{'correct_cert_modulus_hex'} eq $rv->{'cert_modulus_hex'})
+	&&  ($rv->{'correct_cert_exponent_dec'} == $rv->{'cert_exponent_dec'}))
+	{
+		$rv->{'validation'} = 'cert';
+		delete $rv->{'correct_cert_exponent_dec'};
+		delete $rv->{'correct_cert_modulus_hex'};
+		
+		$rv->{'cert_subject_uri'}         = $fp->webid;
+		$rv->{'cert_subject_endpoint'}    = $fp->endpoint;
+		$rv->{'cert_subject_fingerpoint'} = $fp;
+		
+		return 1;
+	}
+
+	return 0;
+}
+
+sub _exec_query 
+{
+	my $rv = shift;
+	my $q  = shift;
+	
+	if (defined $rv->{'cert_subject_model'})
+	{
+		my $Q = RDF::Query->new($q);
+		return $Q->execute($rv->{'cert_subject_model'});
+	}
+	
+	if (defined $rv->{'cert_subject_endpoint'})
+	{
+		my $Q = RDF::Query::Client->new($q);
+		return $Q->execute($rv->{'cert_subject_endpoint'}, {QueryMethod=>'POST'});
+	}
+}
 
 sub load_personal_info
 {
 	my $rv = shift;
 	
-	return 0 unless ((defined $rv) && ($rv->{'validation'} eq 'cert'));
+	return 0
+		unless defined $rv and $rv->{'validation'} eq 'cert';
 	
 	# List of RDF classes that the module understands.
 	my @agentTypes   = ('http://xmlns.com/foaf/0.1/Person',
@@ -335,12 +410,11 @@ sub load_personal_info
 	                          ."    <%s> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?type . "
 	                          ."}",
 	                          $rv->{'cert_subject_uri'});
-	my $query   = RDF::Redland::Query->new($query_string, RDF::Redland::URI->new($rv->{'cert_subject_uri'}), undef, "sparql");
-	my $results = $rv->{'cert_subject_model'}->query_execute($query);
+	my $results = $rv->_exec_query( $query_string );
 	
-	RESULT: while (!$results->finished)
+	RESULT: while (my $row = $results->next)
 	{
-		my $type = $results->binding_value(0)->uri->as_string;
+		my $type = $row->{'type'}->uri;
 		
 		foreach my $t (@agentTypes)
 		{
@@ -376,16 +450,14 @@ sub load_personal_info
 	if (!defined $rv->{'cert_subject_type'}
 	||  $rv->{'cert_subject_type'} eq 'OnlineAccount')
 	{
-		my $query = RDF::Redland::Query->new(
+		my $results = $rv->_exec_query(
 			sprintf('SELECT ?person WHERE { <%s> <http://rdfs.org/sioc/ns#account_of> ?person . }', $rv->{'cert_subject_uri'}),
-			RDF::Redland::URI->new($rv->{'cert_subject_uri'}),
-			undef,
-			'sparql');
-		my $results = $rv->{'cert_subject_model'}->query_execute($query);
-		if (!$results->finished)
+			$rv->{'cert_subject_uri'});
+			
+		if (my $row = $results->next)
 		{
 			$rv->{'cert_subject_type'} = 'OnlineAccount';
-			$retrievedAgentUri = $results->binding_value(0)->uri->as_string
+			$retrievedAgentUri = $row->{'person'}->uri
 				unless defined $retrievedAgentUri;
 		}
 	}
@@ -393,41 +465,35 @@ sub load_personal_info
 	if (!defined $rv->{'cert_subject_type'}
 	||  $rv->{'cert_subject_type'} eq 'OnlineAccount')
 	{
-		my $query = RDF::Redland::Query->new(
+		my $results = $rv->_exec_query(
 			sprintf('SELECT ?person WHERE { ?person <http://xmlns.com/foaf/0.1/holdsAccount> <%s> . }', $rv->{'cert_subject_uri'}),
-			RDF::Redland::URI->new($rv->{'cert_subject_uri'}),
-			undef,
-			'sparql');
-		my $results = $rv->{'cert_subject_model'}->query_execute($query);
-		if (!$results->finished)
+			$rv->{'cert_subject_uri'});
+			
+		if (my $row = $results->next)
 		{
 			$rv->{'cert_subject_type'} = 'OnlineAccount';
-			$retrievedAgentUri = $results->binding_value(0)->uri->as_string
+			$retrievedAgentUri = $row->{'person'}->uri
 				unless defined $retrievedAgentUri;
 		}
 	}
 
 	unless (defined $rv->{'cert_subject_type'})
 	{
-		my $query = RDF::Redland::Query->new(
+		my $results = $rv->_exec_query(
 			sprintf('ASK WHERE { <%s> <http://xmlns.com/foaf/0.1/accountName> ?o . }', $rv->{'cert_subject_uri'}),
-			RDF::Redland::URI->new($rv->{'cert_subject_uri'}),
-			undef,
-			'sparql');
-		my $results = $rv->{'cert_subject_model'}->query_execute($query);
-		if ($results->get_boolean)
+			$rv->{'cert_subject_uri'});
+		
+		if ($results->is_boolean && $results->get_boolean)
 			{ $rv->{'cert_subject_type'} = 'OnlineAccount'; }
 	}
 
 	unless (defined $rv->{'cert_subject_type'})
 	{
-		my $query = RDF::Redland::Query->new(
+		my $results = $rv->_exec_query(
 			sprintf('ASK WHERE { <%s> <http://xmlns.com/foaf/0.1/accountServiceHomepage> ?o . }', $rv->{'cert_subject_uri'}),
-			RDF::Redland::URI->new($rv->{'cert_subject_uri'}),
-			undef,
-			'sparql');
-		my $results = $rv->{'cert_subject_model'}->query_execute($query);
-		if ($results->get_boolean)
+			$rv->{'cert_subject_uri'});
+
+		if ($results->is_boolean && $results->get_boolean)
 			{ $rv->{'cert_subject_type'} = 'OnlineAccount'; }
 	}
 
@@ -436,7 +502,8 @@ sub load_personal_info
 	{
 		$rv->{'agent'} = CGI::Auth::FOAF_SSL::Agent->new(
 			$rv->{'cert_subject_uri'},
-			$rv->{'cert_subject_model'});
+			$rv->{'cert_subject_model'},
+			$rv->{'cert_subject_endpoint'});
 			
 		$rv->{'thing'} = $rv->{'agent'};
 		$rv->{'validation'} = 'agent';
@@ -449,25 +516,24 @@ sub load_personal_info
 	{
 		$rv->{'account'} = CGI::Auth::FOAF_SSL::OnlineAccount->new(
 			$rv->{'cert_subject_uri'},
-			$rv->{'cert_subject_model'});
+			$rv->{'cert_subject_model'},
+			$rv->{'cert_subject_endpoint'});
 
 		$rv->{'thing'} = $rv->{'account'};
 
 		if (defined $retrievedAgentUri)
 		{
-			my $model = $rv->_get_redland_model($retrievedAgentUri);
+			my $model = $rv->get_trine_model($retrievedAgentUri);
 			$rv->{'agent'} = CGI::Auth::FOAF_SSL::Agent->new($retrievedAgentUri, $model);
 			
 			if ($model)
 			{
-				my $query = RDF::Redland::Query->new(
+				my $query = RDF::Query->new(
 					sprintf('ASK WHERE { <%s> <http://xmlns.com/foaf/0.1/holdsAccount> <%s> . }',
 						$retrievedAgentUri, $rv->{'cert_subject_uri'}),
-					RDF::Redland::URI->new($retrievedAgentUri),
-					undef,
-					'sparql');
-				my $results = $model->query_execute($query);
-				if ($results->get_boolean)
+					$retrievedAgentUri);
+				my $results = $query->execute($model);
+				if ($results->is_boolean && $results->get_boolean)
 				{
 					$rv->{'validation'} = 'agent';
 				}
@@ -479,11 +545,34 @@ sub load_personal_info
 
 	$rv->{'thing'} = CGI::Auth::FOAF_SSL::CertifiedThing->new(
 		$rv->{'cert_subject_uri'},
-		$rv->{'cert_subject_model'});
+		$rv->{'cert_subject_model'},
+		$rv->{'cert_subject_endpoint'});
 
 	return 0;
 }
 
+=back
+
+=head1 PUBLIC METHODS
+
+=over 8
+
+=cut
+
+=item $cookie = $auth->cookie
+
+HTTP cookie related to the authentication process. Sending this to the
+client isn't strictly necessary, but it allows for a session to be
+established, greatly speeding up subsequent accesses.
+
+=cut
+
+sub cookie
+{
+	my $this = shift;
+	return $this->{'session'}->cookie;
+}
+
 =item $bool = $auth->is_secure
 
 Returns true iff the authentication process was completely successful.
@@ -545,80 +634,71 @@ sub certified_thing
 
 =back
 
-=head1 INTERNAL METHODS
+=head1 UTILITY METHOD
 
 =over 8
 
-=item $model = $auth->_get_redland_model($uri);
+=item $model = $auth->get_trine_model($uri);
 
-Get an RDF::Redland::Model corresponding to a URI.
+Get an RDF::Trine::Model corresponding to a URI.
 
 =cut
 
-sub _get_redland_model
+sub get_trine_model
 {
 	my $this = shift;
 	my $uri  = shift;
-
-	my $ua = LWP::UserAgent->new(agent=>$CGI::Auth::FOAF_SSL::ua_string); 
-	$ua->default_headers->push_header('Accept' => "application/rdf+xml, application/xhtml+xml, text/html, */*");
-	my $response = $ua->get($uri);
-	return unless length $response->content;
 	
-	my $storage  = RDF::Redland::Storage->new("hashes", "test", "new='yes',hash-type='memory'");
-	my $model    = RDF::Redland::Model->new($storage, "");	
-	
-	# If response is (X)HTML, parse using RDF::RDFa::Parser instead of Redland.
-	if ($response->header('content-type')
-		=~ m#^(text/html|application/xhtml.xml|image/svg(.xml)?)#i)
+	# Session for caching data into.
+	unless (defined $this->{'session'})
 	{
-		my $parser = RDF::RDFa::Parser->new($response->content, $uri);
-		$parser->consume;
-		my $graph = $parser->graph;
-		
-		foreach my $subject (keys %$graph)
-		{
-			my $S = ($subject =~ /^_:(.+)$/) 
-			      ? RDF::Redland::BlankNode->new($1)
-			      : RDF::Redland::URINode->new($subject);
-		
-			foreach my $predicate (keys %{ $graph->{$subject} })
-			{
-				my $P = RDF::Redland::URINode->new($predicate);
-		
-				foreach my $object (@{ $graph->{$subject}->{$predicate} })
-				{
-					my $O;
-					
-					if ($object->{'type'} eq 'literal')
-					{
-						$O = RDF::Redland::LiteralNode->new(
-							$object->{'value'},
-							$object->{'datatype'},
-							$object->{'lang'});
-					}
-					else
-					{
-						$O = ($object->{'value'} =~ /^_:(.+)$/) 
-						      ? RDF::Redland::BlankNode->new($1)
-						      : RDF::Redland::URINode->new($object->{'value'});
-					}
-					
-					$model->add($S, $P, $O);
-				}
-			}
-		}
-		
+		$this->{'session'} = CGI::Session->new('driver:file', undef, {Directory=>'/tmp'});
+		$this->{'session'}->expire('+1h');
+	}
+	
+	# Check to see if this URI has already been retrieved.
+	if (defined $this->{'session'}->param($uri)
+	and length $this->{'session'}->param($uri))
+	{
+		my $parser = RDF::Trine::Parser::Turtle->new;
+		my $model  = RDF::Trine::Model->new( RDF::Trine::Store->temporary_store );
+		$parser->parse_into_model( $uri, $this->{'session'}->param($uri) , $model );
 		return $model;
 	}
 	
-	my $URI      = RDF::Redland::URI->new($uri);
-	my $parser   = RDF::Redland::Parser->new(undef, $response->header('content-type'));
+	my $ua = LWP::UserAgent->new(agent=>$CGI::Auth::FOAF_SSL::ua_string); 
+	$ua->default_headers->push_header('Accept' => "application/rdf+xml, application/xhtml+xml, text/html, text/turtle, application/x-turtle, */*");
+	my $response = $ua->get($uri);
+	return unless length $response->content;
 	
-	$parser = RDF::Redland::Parser->new('rdfxml')
-		unless ($parser);
+	my $model;
+	
+	# If response is (X)HTML, parse using RDF::RDFa::Parser instead of Trine.
+	if ($response->header('content-type')
+		=~ m#^(text/html|application/xhtml.xml|image/svg(.xml)?)#i)
+	{
+		my $opts = $response->header('content-type') =~ m#^(image/svg(.xml)?)#i
+			? RDF::RDFa::Parser::OPTS_SVG
+			: RDF::RDFa::Parser::OPTS_XHTML ;
+		my $parser = RDF::RDFa::Parser->new($response->decoded_content, $uri, $opts);
+		$parser->consume;
+		$model = $parser->graph;
+	}
 
-	$parser->parse_string_into_model($response->content, $URI, $model);	
+	# Not RDFa, so let Trine handle it as best it can.
+	else
+	{
+			$model      = RDF::Trine::Model->new( RDF::Trine::Store->temporary_store );
+			my $parser  = ($response->header('content-type') =~ /(n3|turtle|text.plain)/i)
+							? RDF::Trine::Parser::Turtle->new
+							: RDF::Trine::Parser::RDFXML->new;
+			$parser->parse_into_model( $uri , $response->decoded_content , $model );
+	}
+	
+	my $serializer = RDF::Trine::Serializer::NTriples->new();
+	$this->{'session'}->param($uri,
+		$serializer->serialize_model_to_string($model));
+	$this->{'session'}->flush;
 	
 	return $model;
 }
@@ -629,24 +709,31 @@ __END__
 
 =back
 
+=head1 BUGS
+
+Please report any bugs to L<http://rt.cpan.org/>.
+
 =head1 SEE ALSO
 
-L<CGI>
+L<http://lists.foaf-project.org/mailman/listinfo/foaf-protocols>,
+L<http://esw.w3.org/topic/foaf+ssl>.
 
-L<http://esw.w3.org/topic/foaf+ssl>
+L<CGI>, L<RDF::Trine>.
 
-L<http://httpd.apache.org/docs/2.0/mod/mod_ssl.html>
+L<http://httpd.apache.org/docs/2.0/mod/mod_ssl.html>.
+
+L<http://www.perlrdf.org/>.
 
 =head1 AUTHOR
 
-Toby Inkster, E<lt>mail@tobyinkster.co.ukE<gt>
+Toby Inkster, E<lt>tobyink@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2009 by Toby Inkster
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.10.0 or,
+it under the same terms as Perl itself, either Perl version 5.8.1 or,
 at your option, any later version of Perl 5 you may have available.
 
 =cut
